﻿namespace POKEMON_BW2
{
    using System;
    using System.Reflection;

    public class PokemonData
    {
        private byte[] data = new byte[0x4c];
        public const int MACHINE_SIZE = 13;
        public const int SIZE = 0x4c;

        private int ByteOfInt(int value)
        {
            if (value > 0xff)
            {
                value = 0xff;
            }
            else if (value < -256)
            {
                value = -256;
            }
            return (value & 0xff);
        }

        public int GetMachine(int index)
        {
            this.MachineIndexCheck(index);
            return this.data[0x1c + index];
        }

        public int[] GetMachineArray()
        {
            int[] numArray = new int[13];
            for (int i = 0; i < numArray.Length; i++)
            {
                numArray[i] = this.GetMachine(i);
            }
            return numArray;
        }

        public int GetMachineLength()
        {
            return 13;
        }

        public bool IsMachineUsable(int index)
        {
            int num = index / 8;
            int num2 = index % 8;
            int num3 = (this.data[40 + num] >> num2) & 1;
            if (num3 != 1)
            {
                return false;
            }
            return true;
        }

        private void MachineIndexCheck(int index)
        {
            if ((index < 0) || (index >= 13))
            {
                throw new IndexOutOfRangeException();
            }
        }

        public void SetMachine(int index, int value)
        {
            this.MachineIndexCheck(index);
            this.data[0x1c + index] = (byte) this.ByteOfInt(value);
        }

        public void SetMachineArray(int[] machine)
        {
            for (int i = 0; i < machine.Length; i++)
            {
                this.SetMachine(i, machine[i]);
            }
        }

        public void SetMachineUsable(int index, bool value)
        {
            int num = index / 8;
            int num2 = index % 8;
            int num3 = 0;
            int num4 = 0;
            switch (num2)
            {
                case 0:
                    num3 += value ? 1 : 0;
                    num4 = 0xfe;
                    break;

                case 1:
                    num3 += value ? 2 : 0;
                    num4 = 0xfd;
                    break;

                case 2:
                    num3 += value ? 4 : 0;
                    num4 = 0xfb;
                    break;

                case 3:
                    num3 += value ? 8 : 0;
                    num4 = 0xf7;
                    break;

                case 4:
                    num3 += value ? 0x10 : 0;
                    num4 = 0xef;
                    break;

                case 5:
                    num3 += value ? 0x20 : 0;
                    num4 = 0xdf;
                    break;

                case 6:
                    num3 += value ? 0x40 : 0;
                    num4 = 0xbf;
                    break;

                case 7:
                    num3 += value ? 0x80 : 0;
                    num4 = 0x7f;
                    break;
            }
            this.data[40 + num] = (byte) (num3 + (this.data[40 + num] & num4));
        }

        private int ShortOfInt(int value)
        {
            if (value > 0xffff)
            {
                value = 0xffff;
            }
            else if (value < -65535)
            {
                value = -65535;
            }
            return (value & 0xffff);
        }

        public int BaseExp
        {
            get
            {
                return (this.data[0x22] + (this.data[0x23] * 0x100));
            }
            set
            {
                this.data[0x22] = (byte) (this.ShortOfInt(value) & 0xff);
                this.data[0x23] = (byte) ((this.ShortOfInt(value) >> 8) & 0xff);
            }
        }

        public int Capture
        {
            get
            {
                return this.data[8];
            }
            set
            {
                this.data[8] = (byte) this.ByteOfInt(value);
            }
        }

        public byte[] Data
        {
            get
            {
                return this.data;
            }
            set
            {
                this.data = value;
            }
        }

        public int Effort
        {
            get
            {
                return (this.data[10] + (this.data[11] * 0x100));
            }
            set
            {
                this.data[10] = (byte) (this.ShortOfInt(value) & 0xff);
                this.data[11] = (byte) ((this.ShortOfInt(value) >> 8) & 0xff);
            }
        }

        public int Effort1
        {
            get
            {
                return this.data[10];
            }
            set
            {
                this.data[10] = (byte) this.ByteOfInt(value);
            }
        }

        public int Effort2
        {
            get
            {
                return this.data[11];
            }
            set
            {
                this.data[11] = (byte) this.ByteOfInt(value);
            }
        }

        public int EffortAtk
        {
            get
            {
                return ((this.data[10] >> 2) & 3);
            }
            set
            {
                int num = (this.ByteOfInt(value) & 3) << 2;
                int num2 = this.data[10] & 0xf3;
                this.data[10] = (byte) (num + num2);
            }
        }

        public int EffortDef
        {
            get
            {
                return ((this.data[10] >> 4) & 3);
            }
            set
            {
                int num = (this.ByteOfInt(value) & 3) << 4;
                int num2 = this.data[10] & 0xcf;
                this.data[10] = (byte) (num + num2);
            }
        }

        public int EffortHp
        {
            get
            {
                return (this.data[10] & 3);
            }
            set
            {
                int num = this.ByteOfInt(value) & 3;
                int num2 = this.data[10] & 0xfc;
                this.data[10] = (byte) (num + num2);
            }
        }

        public int EffortSatk
        {
            get
            {
                return (this.data[11] & 3);
            }
            set
            {
                int num = this.ByteOfInt(value) & 3;
                int num2 = this.data[11] & 0xfc;
                this.data[11] = (byte) (num + num2);
            }
        }

        public int EffortSdef
        {
            get
            {
                return ((this.data[11] >> 2) & 3);
            }
            set
            {
                int num = (this.ByteOfInt(value) & 3) << 2;
                int num2 = this.data[11] & 0xf3;
                this.data[11] = (byte) (num + num2);
            }
        }

        public int EffortSpd
        {
            get
            {
                return ((this.data[10] >> 6) & 3);
            }
            set
            {
                int num = (this.ByteOfInt(value) & 3) << 6;
                int num2 = this.data[10] & 0x3f;
                this.data[10] = (byte) (num + num2);
            }
        }

        public int EggStep
        {
            get
            {
                return this.data[0x13];
            }
            set
            {
                this.data[0x13] = (byte) this.ByteOfInt(value);
            }
        }

        public int Escape
        {
            get
            {
                return this.data[0x1b];
            }
            set
            {
                this.data[0x1b] = (byte) this.ByteOfInt(value);
            }
        }

        public int ExpType
        {
            get
            {
                return this.data[0x15];
            }
            set
            {
                this.data[0x15] = (byte) this.ByteOfInt(value);
            }
        }

        public int FamilyAtk
        {
            get
            {
                return this.data[1];
            }
            set
            {
                this.data[1] = (byte) this.ByteOfInt(value);
            }
        }

        public int FamilyDef
        {
            get
            {
                return this.data[2];
            }
            set
            {
                this.data[2] = (byte) this.ByteOfInt(value);
            }
        }

        public int FamilyHp
        {
            get
            {
                return this.data[0];
            }
            set
            {
                this.data[0] = (byte) this.ByteOfInt(value);
            }
        }

        public int FamilySatk
        {
            get
            {
                return this.data[4];
            }
            set
            {
                this.data[4] = (byte) this.ByteOfInt(value);
            }
        }

        public int FamilySdef
        {
            get
            {
                return this.data[5];
            }
            set
            {
                this.data[5] = (byte) this.ByteOfInt(value);
            }
        }

        public int FamilySpd
        {
            get
            {
                return this.data[3];
            }
            set
            {
                this.data[3] = (byte) this.ByteOfInt(value);
            }
        }

        public int Feel
        {
            get
            {
                return this.data[20];
            }
            set
            {
                this.data[20] = (byte) this.ByteOfInt(value);
            }
        }

        public int Group1
        {
            get
            {
                return this.data[0x16];
            }
            set
            {
                this.data[0x16] = (byte) this.ByteOfInt(value);
            }
        }

        public int Group2
        {
            get
            {
                return this.data[0x17];
            }
            set
            {
                this.data[0x17] = (byte) this.ByteOfInt(value);
            }
        }

        public byte this[int i]
        {
            get
            {
                return this.data[i];
            }
            set
            {
                this.data[i] = value;
            }
        }

        public int Item1
        {
            get
            {
                return (this.data[12] + (this.data[13] * 0x100));
            }
            set
            {
                this.data[12] = (byte) (this.ShortOfInt(value) & 0xff);
                this.data[13] = (byte) ((this.ShortOfInt(value) >> 8) & 0xff);
            }
        }

        public int Item1A
        {
            get
            {
                return this.data[12];
            }
            set
            {
                this.data[12] = (byte) this.ByteOfInt(value);
            }
        }

        public int Item1B
        {
            get
            {
                return this.data[13];
            }
            set
            {
                this.data[13] = (byte) this.ByteOfInt(value);
            }
        }

        public int Item2
        {
            get
            {
                return (this.data[14] + (this.data[15] * 0x100));
            }
            set
            {
                this.data[14] = (byte) (this.ShortOfInt(value) & 0xff);
                this.data[15] = (byte) ((this.ShortOfInt(value) >> 8) & 0xff);
            }
        }

        public int Item2A
        {
            get
            {
                return this.data[14];
            }
            set
            {
                this.data[14] = (byte) this.ByteOfInt(value);
            }
        }

        public int Item2B
        {
            get
            {
                return this.data[15];
            }
            set
            {
                this.data[15] = (byte) this.ByteOfInt(value);
            }
        }

        public int Item3
        {
            get
            {
                return (this.data[0x10] + (this.data[0x11] * 0x100));
            }
            set
            {
                this.data[0x10] = (byte) (this.ShortOfInt(value) & 0xff);
                this.data[0x11] = (byte) ((this.ShortOfInt(value) >> 8) & 0xff);
            }
        }

        public int Item3A
        {
            get
            {
                return this.data[0x10];
            }
            set
            {
                this.data[0x10] = (byte) this.ByteOfInt(value);
            }
        }

        public int Item3B
        {
            get
            {
                return this.data[0x11];
            }
            set
            {
                this.data[0x11] = (byte) this.ByteOfInt(value);
            }
        }

        public int Length
        {
            get
            {
                return this.data.Length;
            }
        }

        public int Look
        {
            get
            {
                return this.data[0x21];
            }
            set
            {
                this.data[0x21] = (byte) this.ByteOfInt(value);
            }
        }

        public int Rank
        {
            get
            {
                return this.data[9];
            }
            set
            {
                this.data[9] = (byte) this.ByteOfInt(value);
            }
        }

        public int Sex
        {
            get
            {
                return this.data[0x12];
            }
            set
            {
                this.data[0x12] = (byte) this.ByteOfInt(value);
            }
        }

        public int Specific1
        {
            get
            {
                return this.data[0x18];
            }
            set
            {
                this.data[0x18] = (byte) this.ByteOfInt(value);
            }
        }

        public int Specific2
        {
            get
            {
                return this.data[0x19];
            }
            set
            {
                this.data[0x19] = (byte) this.ByteOfInt(value);
            }
        }

        public int Specific3
        {
            get
            {
                return this.data[0x1a];
            }
            set
            {
                this.data[0x1a] = (byte) this.ByteOfInt(value);
            }
        }

        public int Type1
        {
            get
            {
                return this.data[6];
            }
            set
            {
                this.data[6] = (byte) this.ByteOfInt(value);
            }
        }

        public int Type2
        {
            get
            {
                return this.data[7];
            }
            set
            {
                this.data[7] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown1C
        {
            get
            {
                return this.data[0x1c];
            }
            set
            {
                this.data[0x1c] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown1D
        {
            get
            {
                return this.data[0x1d];
            }
            set
            {
                this.data[0x1d] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown1E
        {
            get
            {
                return this.data[30];
            }
            set
            {
                this.data[30] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown1F
        {
            get
            {
                return this.data[0x1f];
            }
            set
            {
                this.data[0x1f] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown20
        {
            get
            {
                return this.data[0x20];
            }
            set
            {
                this.data[0x20] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown24
        {
            get
            {
                return this.data[0x24];
            }
            set
            {
                this.data[0x24] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown25
        {
            get
            {
                return this.data[0x25];
            }
            set
            {
                this.data[0x25] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown26
        {
            get
            {
                return this.data[0x26];
            }
            set
            {
                this.data[0x26] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown27
        {
            get
            {
                return this.data[0x27];
            }
            set
            {
                this.data[0x27] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown35
        {
            get
            {
                return this.data[0x35];
            }
            set
            {
                this.data[0x35] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown36
        {
            get
            {
                return this.data[0x36];
            }
            set
            {
                this.data[0x36] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown37
        {
            get
            {
                return this.data[0x37];
            }
            set
            {
                this.data[0x37] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown38
        {
            get
            {
                return this.data[0x38];
            }
            set
            {
                this.data[0x38] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown39
        {
            get
            {
                return this.data[0x39];
            }
            set
            {
                this.data[0x39] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown3A
        {
            get
            {
                return this.data[0x3a];
            }
            set
            {
                this.data[0x3a] = (byte) this.ByteOfInt(value);
            }
        }

        public int Unkown3B
        {
            get
            {
                return this.data[0x3b];
            }
            set
            {
                this.data[0x3b] = (byte) this.ByteOfInt(value);
            }
        }
    }
}

